@@ -20,35 +20,22 @@
  */
 
 #include "config.h"
-#include <string.h>
-#include <gdk/gdkkeysyms.h>
-#include <gtk/gtk.h>
-
 #include "dh-book-tree.h"
 #include "dh-book.h"
 
 typedef struct {
         const gchar *uri;
-        gboolean     found;
         GtkTreeIter  iter;
         GtkTreePath *path;
+        guint        found : 1;
 } FindURIData;
 
 typedef struct {
         GtkTreeStore  *store;
         DhBookManager *book_manager;
         DhLink        *selected_link;
-
-        /* Signals */
-        guint book_created_id;
-        guint book_deleted_id;
-        guint book_enabled_id;
-        guint book_disabled_id;
-        guint group_by_language_id;
 } DhBookTreePrivate;
 
-static void dh_book_tree_class_init        (DhBookTreeClass  *klass);
-static void dh_book_tree_init              (DhBookTree       *tree);
 static void book_tree_add_columns          (DhBookTree       *tree);
 static void book_tree_setup_selection      (DhBookTree       *tree);
 static void book_tree_insert_node          (DhBookTree       *tree,
@@ -64,12 +51,12 @@ enum {
 };
 
 enum {
-	COL_TITLE,
-	COL_LINK,
+        COL_TITLE,
+        COL_LINK,
         COL_BOOK,
-	COL_WEIGHT,
+        COL_WEIGHT,
         COL_UNDERLINE,
-	N_COLUMNS
+        N_COLUMNS
 };
 
 enum {
@@ -86,37 +73,6 @@ dh_book_tree_dispose (GObject *object)
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (DH_BOOK_TREE (object));
 
-        /* Disconnect signals */
-        if (priv->book_created_id != 0 &&
-            g_signal_handler_is_connected (priv->book_manager, priv->book_created_id)) {
-                g_signal_handler_disconnect (priv->book_manager, priv->book_created_id);
-                priv->book_created_id = 0;
-        }
-
-        if (priv->book_deleted_id &&
-            g_signal_handler_is_connected (priv->book_manager, priv->book_deleted_id)) {
-                g_signal_handler_disconnect (priv->book_manager, priv->book_deleted_id);
-                priv->book_deleted_id = 0;
-        }
-
-        if (priv->book_enabled_id != 0 &&
-            g_signal_handler_is_connected (priv->book_manager, priv->book_enabled_id)) {
-                g_signal_handler_disconnect (priv->book_manager, priv->book_enabled_id);
-                priv->book_enabled_id = 0;
-        }
-
-        if (priv->book_disabled_id != 0 &&
-            g_signal_handler_is_connected (priv->book_manager, priv->book_disabled_id)) {
-                g_signal_handler_disconnect (priv->book_manager, priv->book_disabled_id);
-                priv->book_disabled_id = 0;
-        }
-
-        if (priv->group_by_language_id != 0 &&
-            g_signal_handler_is_connected (priv->book_manager, priv->group_by_language_id)) {
-                g_signal_handler_disconnect (priv->book_manager, priv->group_by_language_id);
-                priv->group_by_language_id = 0;
-        }
-
         g_clear_object (&priv->store);
         g_clear_object (&priv->book_manager);
 
@@ -167,63 +123,65 @@ dh_book_tree_init (DhBookTree *tree)
 
         priv = dh_book_tree_get_instance_private (tree);
 
-	priv->store = gtk_tree_store_new (N_COLUMNS,
-					  G_TYPE_STRING,
-					  G_TYPE_POINTER,
+        priv->store = gtk_tree_store_new (N_COLUMNS,
+                                          G_TYPE_STRING,
+                                          G_TYPE_POINTER,
                                           G_TYPE_OBJECT,
                                           PANGO_TYPE_WEIGHT,
                                           PANGO_TYPE_UNDERLINE);
-	priv->selected_link = NULL;
-	gtk_tree_view_set_model (GTK_TREE_VIEW (tree),
-				 GTK_TREE_MODEL (priv->store));
+        priv->selected_link = NULL;
+        gtk_tree_view_set_model (GTK_TREE_VIEW (tree),
+                                 GTK_TREE_MODEL (priv->store));
 
-	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), FALSE);
+        gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), FALSE);
 
-	book_tree_add_columns (tree);
+        book_tree_add_columns (tree);
 
-	book_tree_setup_selection (tree);
+        book_tree_setup_selection (tree);
 }
 
 static void
 book_tree_add_columns (DhBookTree *tree)
 {
-	GtkCellRenderer   *cell;
-	GtkTreeViewColumn *column;
+        GtkCellRenderer   *cell;
+        GtkTreeViewColumn *column;
 
-	column = gtk_tree_view_column_new ();
+        column = gtk_tree_view_column_new ();
 
-	cell = gtk_cell_renderer_text_new ();
-	g_object_set (cell,
-		      "ellipsize", PANGO_ELLIPSIZE_END,
-		      NULL);
-	gtk_tree_view_column_pack_start (column, cell, TRUE);
-	gtk_tree_view_column_set_attributes (column, cell,
-					     "text", COL_TITLE,
+        cell = gtk_cell_renderer_text_new ();
+        g_object_set (cell,
+                      "ellipsize", PANGO_ELLIPSIZE_END,
+                      NULL);
+        gtk_tree_view_column_pack_start (column, cell, TRUE);
+        gtk_tree_view_column_set_attributes (column, cell,
+                                             "text", COL_TITLE,
                                              "weight", COL_WEIGHT,
                                              "underline", COL_UNDERLINE,
-					     NULL);
+                                             NULL);
 
-	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
+        gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
 }
 
 static void
 book_tree_setup_selection (DhBookTree *tree)
 {
-	GtkTreeSelection *selection;
+        GtkTreeSelection *selection;
 
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
 
-	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
+        gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
 
-	g_signal_connect (selection, "changed",
-			  G_CALLBACK (book_tree_selection_changed_cb),
-			  tree);
+        g_signal_connect_object (selection,
+                                 "changed",
+                                 G_CALLBACK (book_tree_selection_changed_cb),
+                                 tree,
+                                 0);
 }
 
 /* Tries to find:
  *  - An exact match of the language group
- *  - The language group which should be just after our given language group.
- *  - Both.
+ *  - Or the language group which should be just after our given language group.
+ *  - Or both.
  */
 static void
 book_tree_find_language_group (DhBookTree  *tree,
@@ -255,7 +213,7 @@ book_tree_find_language_group (DhBookTree  *tree,
         }
 
         do {
-                gchar  *title = NULL;
+                gchar *title = NULL;
 
                 /* Look for language titles, which are those where there
                  * is no book object associated in the row */
@@ -291,8 +249,8 @@ book_tree_find_language_group (DhBookTree  *tree,
 /* Tries to find, starting at 'first' (if given), and always in the same
  * level of the tree:
  *  - An exact match of the book
- *  - The book which should be just after our given book
- *  - Both.
+ *  - Or the book which should be just after our given book
+ *  - Or both.
  */
 static void
 book_tree_find_book (DhBookTree        *tree,
@@ -596,21 +554,21 @@ book_tree_book_deleted_or_disabled_cb (DhBookManager *book_manager,
 
 static void
 book_tree_insert_node (DhBookTree  *tree,
-		       GNode       *node,
+                       GNode       *node,
                        GtkTreeIter *current_iter,
                        DhBook      *book)
 
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-	DhLink         *link;
+        DhLink         *link;
         PangoWeight     weight;
-	GNode          *child;
+        GNode          *child;
 
-	link = node->data;
+        link = node->data;
 
-	if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK) {
+        if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK) {
                 weight = PANGO_WEIGHT_BOLD;
-	} else {
+        } else {
                 weight = PANGO_WEIGHT_NORMAL;
         }
 
@@ -623,29 +581,29 @@ book_tree_insert_node (DhBookTree  *tree,
                             COL_UNDERLINE, PANGO_UNDERLINE_NONE,
                             -1);
 
-	for (child = g_node_first_child (node);
-	     child;
-	     child = g_node_next_sibling (child)) {
+        for (child = g_node_first_child (node);
+             child;
+             child = g_node_next_sibling (child)) {
                 GtkTreeIter iter;
 
                 /* Append new iter */
                 gtk_tree_store_append (priv->store, &iter, current_iter);
-		book_tree_insert_node (tree, child, &iter, NULL);
-	}
+                book_tree_insert_node (tree, child, &iter, NULL);
+        }
 }
 
 static void
 book_tree_selection_changed_cb (GtkTreeSelection *selection,
-				DhBookTree       *tree)
+                                DhBookTree       *tree)
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
         GtkTreeIter     iter;
 
-	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+        if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
                 DhLink *link;
 
-		gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
-				    &iter,
+                gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
+                                    &iter,
                                     COL_LINK, &link,
                                     -1);
                 if (link) {
@@ -654,7 +612,7 @@ book_tree_selection_changed_cb (GtkTreeSelection *selection,
                         }
                         priv->selected_link = link;
                 }
-	}
+        }
 }
 
 static void
@@ -665,7 +623,6 @@ book_tree_group_by_language_cb (GObject    *object,
         book_tree_populate_tree (DH_BOOK_TREE (user_data));
 }
 
-
 static void
 book_tree_init_selection (DhBookTree *tree)
 {
@@ -677,17 +634,17 @@ book_tree_init_selection (DhBookTree *tree)
 
         priv = dh_book_tree_get_instance_private (tree);
 
-	/* Mark the first item as selected, or it would get automatically
-	 * selected when the treeview will get focus; but that's not even
-	 * enough as a selection changed would still be emitted when there
-	 * is no change, hence the manual tracking of selection in
-	 * selected_link.
-	 *   https://bugzilla.gnome.org/show_bug.cgi?id=492206
-	 */
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
-	g_signal_handlers_block_by_func	(selection,
-					 book_tree_selection_changed_cb,
-					 tree);
+        /* Mark the first item as selected, or it would get automatically
+         * selected when the treeview will get focus; but that's not even
+         * enough as a selection changed would still be emitted when there
+         * is no change, hence the manual tracking of selection in
+         * selected_link.
+         *   https://bugzilla.gnome.org/show_bug.cgi?id=492206
+         */
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+        g_signal_handlers_block_by_func (selection,
+                                         book_tree_selection_changed_cb,
+                                         tree);
 
         /* If grouping by languages, get first book in the first language */
         if (dh_book_manager_get_group_by_language (priv->book_manager)) {
@@ -713,9 +670,9 @@ book_tree_init_selection (DhBookTree *tree)
                 gtk_tree_selection_select_iter (selection, &iter);
         }
 
-	g_signal_handlers_unblock_by_func (selection,
-					   book_tree_selection_changed_cb,
-					   tree);
+        g_signal_handlers_unblock_by_func (selection,
+                                           book_tree_selection_changed_cb,
+                                           tree);
 }
 
 static void
@@ -724,26 +681,35 @@ dh_book_tree_constructed (GObject *object)
         DhBookTree *tree = DH_BOOK_TREE (object);
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
 
-        priv->book_created_id = g_signal_connect (priv->book_manager,
-                                                  "book-created",
-                                                  G_CALLBACK (book_tree_book_created_or_enabled_cb),
-                                                  tree);
-        priv->book_deleted_id = g_signal_connect (priv->book_manager,
-                                                  "book-deleted",
-                                                  G_CALLBACK (book_tree_book_deleted_or_disabled_cb),
-                                                  tree);
-        priv->book_enabled_id = g_signal_connect (priv->book_manager,
-                                                  "book-enabled",
-                                                  G_CALLBACK (book_tree_book_created_or_enabled_cb),
-                                                  tree);
-        priv->book_disabled_id = g_signal_connect (priv->book_manager,
-                                                   "book-disabled",
-                                                   G_CALLBACK (book_tree_book_deleted_or_disabled_cb),
-                                                   tree);
-        priv->group_by_language_id = g_signal_connect (priv->book_manager,
-                                                       "notify::group-by-language",
-                                                       G_CALLBACK (book_tree_group_by_language_cb),
-                                                       tree);
+        g_signal_connect_object (priv->book_manager,
+                                 "book-created",
+                                 G_CALLBACK (book_tree_book_created_or_enabled_cb),
+                                 tree,
+                                 0);
+
+        g_signal_connect_object (priv->book_manager,
+                                 "book-deleted",
+                                 G_CALLBACK (book_tree_book_deleted_or_disabled_cb),
+                                 tree,
+                                 0);
+
+        g_signal_connect_object (priv->book_manager,
+                                 "book-enabled",
+                                 G_CALLBACK (book_tree_book_created_or_enabled_cb),
+                                 tree,
+                                 0);
+
+        g_signal_connect_object (priv->book_manager,
+                                 "book-disabled",
+                                 G_CALLBACK (book_tree_book_deleted_or_disabled_cb),
+                                 tree,
+                                 0);
+
+        g_signal_connect_object (priv->book_manager,
+                                 "notify::group-by-language",
+                                 G_CALLBACK (book_tree_group_by_language_cb),
+                                 tree,
+                                 0);
 
         book_tree_populate_tree (tree);
 
@@ -766,44 +732,49 @@ dh_book_tree_class_init (DhBookTreeClass *klass)
                                          PROP_BOOK_MANAGER,
                                          g_param_spec_object ("book-manager",
                                                               "Book Manager",
-                                                              "The book maanger",
+                                                              "The book manager",
                                                               DH_TYPE_BOOK_MANAGER,
                                                               G_PARAM_READWRITE |
                                                               G_PARAM_CONSTRUCT_ONLY));
 
         signals[LINK_SELECTED] =
                 g_signal_new ("link-selected",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__POINTER,
-			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+                              G_TYPE_FROM_CLASS (klass),
+                              G_SIGNAL_RUN_LAST,
+                              0,
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__POINTER,
+                              G_TYPE_NONE,
+                              1, G_TYPE_POINTER);
 }
 
 GtkWidget *
 dh_book_tree_new (DhBookManager *book_manager)
 {
-        return GTK_WIDGET (g_object_new (DH_TYPE_BOOK_TREE, "book-manager", book_manager, NULL));
+        return GTK_WIDGET (g_object_new (DH_TYPE_BOOK_TREE,
+                                         "book-manager", book_manager,
+                                         "enable-search", FALSE,
+                                         NULL));
 }
 
 static gboolean
 book_tree_find_uri_foreach (GtkTreeModel *model,
-			    GtkTreePath  *path,
-			    GtkTreeIter  *iter,
-			    FindURIData  *data)
+                            GtkTreePath  *path,
+                            GtkTreeIter  *iter,
+                            FindURIData  *data)
 {
-	DhLink *link;
+        DhLink *link;
 
-	gtk_tree_model_get (model, iter,
-			    COL_LINK, &link,
-			    -1);
+        gtk_tree_model_get (model, iter,
+                            COL_LINK, &link,
+                            -1);
         if (link) {
                 gchar *link_uri;
 
                 link_uri = dh_link_get_uri (link);
-                if (g_str_has_prefix (data->uri, link_uri)) {
+
+                if (link_uri != NULL &&
+                    g_str_has_prefix (data->uri, link_uri)) {
                         data->found = TRUE;
                         data->iter = *iter;
                         data->path = gtk_tree_path_copy (path);
@@ -811,81 +782,81 @@ book_tree_find_uri_foreach (GtkTreeModel *model,
                 g_free (link_uri);
         }
 
-	return data->found;
+        return data->found;
 }
 
 void
 dh_book_tree_select_uri (DhBookTree  *tree,
-			 const gchar *uri)
+                         const gchar *uri)
 {
         DhBookTreePrivate   *priv = dh_book_tree_get_instance_private (tree);
-	GtkTreeSelection *selection;
-	FindURIData       data;
+        GtkTreeSelection *selection;
+        FindURIData       data;
 
-	data.found = FALSE;
-	data.uri = uri;
+        data.found = FALSE;
+        data.uri = uri;
 
-	gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
-				(GtkTreeModelForeachFunc) book_tree_find_uri_foreach,
-				&data);
+        gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
+                                (GtkTreeModelForeachFunc) book_tree_find_uri_foreach,
+                                &data);
 
-	if (!data.found) {
-		return;
-	}
+        if (!data.found) {
+                return;
+        }
 
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
 
         /* Do not re-select (which will expand current additionally) if already
          * there. */
         if (gtk_tree_selection_iter_is_selected (selection, &data.iter))
                 return;
 
-	g_signal_handlers_block_by_func	(selection,
-					 book_tree_selection_changed_cb,
-					 tree);
+        g_signal_handlers_block_by_func (selection,
+                                         book_tree_selection_changed_cb,
+                                         tree);
 
-	gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree), data.path);
-	gtk_tree_selection_select_iter (selection, &data.iter);
-	gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree), data.path, NULL, 0);
+        gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree), data.path);
+        gtk_tree_selection_select_iter (selection, &data.iter);
+        gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree), data.path, NULL, 0);
 
-	g_signal_handlers_unblock_by_func (selection,
-					   book_tree_selection_changed_cb,
-					   tree);
+        g_signal_handlers_unblock_by_func (selection,
+                                           book_tree_selection_changed_cb,
+                                           tree);
 
-	gtk_tree_path_free (data.path);
+        gtk_tree_path_free (data.path);
 }
 
 DhLink *
 dh_book_tree_get_selected_book (DhBookTree *tree)
 {
-	GtkTreeSelection *selection;
-	GtkTreeModel     *model;
-	GtkTreeIter       iter;
-	GtkTreePath      *path;
-	DhLink           *link;
+        GtkTreeSelection *selection;
+        GtkTreeModel     *model;
+        GtkTreeIter       iter;
+        GtkTreePath      *path;
+        DhLink           *link;
 
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
-	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
-		return NULL;
-	}
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+        if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
+                return NULL;
+        }
 
-	path = gtk_tree_model_get_path (model, &iter);
+        path = gtk_tree_model_get_path (model, &iter);
 
-	/* Get the book node for this link. */
-	while (1) {
-		if (gtk_tree_path_get_depth (path) <= 1) {
-			break;
-		}
+        /* Get the book node for this link. */
+        while (TRUE) {
+                if (gtk_tree_path_get_depth (path) <= 1) {
+                        break;
+                }
 
-		gtk_tree_path_up (path);
-	}
+                gtk_tree_path_up (path);
+        }
 
-	gtk_tree_model_get_iter (model, &iter, path);
-	gtk_tree_path_free (path);
+        gtk_tree_model_get_iter (model, &iter, path);
+        gtk_tree_path_free (path);
 
-	gtk_tree_model_get (model, &iter,
-			    COL_LINK, &link,
-			    -1);
+        gtk_tree_model_get (model, &iter,
+                            COL_LINK, &link,
+                            -1);
 
-	return link;
+        return link;
 }
