@@ -21,16 +21,11 @@
  */
 
 #include "config.h"
+#include "dh-sidebar.h"
 
 #include <string.h>
-#include <glib/gi18n-lib.h>
-#include <gdk/gdkkeysyms.h>
-#include <gtk/gtk.h>
 
 #include "dh-keyword-model.h"
-#include "dh-sidebar.h"
-#include "dh-util.h"
-#include "dh-book-manager.h"
 #include "dh-book.h"
 #include "dh-book-tree.h"
 
@@ -48,8 +43,8 @@ typedef struct {
         GtkWidget      *sw_book_tree;
 
         GCompletion    *completion;
-        guint           idle_complete;
-        guint           idle_filter;
+        guint           idle_complete_id;
+        guint           idle_filter_id;
 } DhSidebarPrivate;
 
 enum {
@@ -69,7 +64,7 @@ G_DEFINE_TYPE_WITH_PRIVATE (DhSidebar, dh_sidebar, GTK_TYPE_BOX)
 /******************************************************************************/
 
 static gboolean
-sidebar_filter_idle (DhSidebar *sidebar)
+sidebar_filter_idle_cb (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv;
         const gchar *str;
@@ -78,7 +73,7 @@ sidebar_filter_idle (DhSidebar *sidebar)
 
         priv = dh_sidebar_get_instance_private (sidebar);
 
-        priv->idle_filter = 0;
+        priv->idle_filter_id = 0;
 
         str = gtk_entry_get_text (GTK_ENTRY (priv->entry));
 
@@ -92,7 +87,7 @@ sidebar_filter_idle (DhSidebar *sidebar)
         if (link)
                 g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
 
-        return FALSE;
+        return G_SOURCE_REMOVE;
 }
 
 static void
@@ -100,9 +95,9 @@ sidebar_search_run_idle (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
-        if (!priv->idle_filter)
-                priv->idle_filter =
-                        g_idle_add ((GSourceFunc) sidebar_filter_idle, sidebar);
+        if (priv->idle_filter_id == 0)
+                priv->idle_filter_id =
+                        g_idle_add ((GSourceFunc) sidebar_filter_idle_cb, sidebar);
 }
 
 /******************************************************************************/
@@ -209,7 +204,7 @@ sidebar_tree_button_press_cb (GtkTreeView    *view,
         gtk_tree_view_get_path_at_pos (view, event->x, event->y, &path,
                                        NULL, NULL, NULL);
         if (!path)
-                return FALSE;
+                return GDK_EVENT_PROPAGATE;
 
         gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->model), &iter, path);
         gtk_tree_path_free (path);
@@ -223,10 +218,10 @@ sidebar_tree_button_press_cb (GtkTreeView    *view,
 
         g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
 
-        /* Always return FALSE so the tree view gets the event and can update
+        /* Always propagate the event so the tree view can update
          * the selection etc.
          */
-        return FALSE;
+        return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -243,7 +238,8 @@ sidebar_entry_key_press_event_cb (GtkEntry    *entry,
                         gtk_editable_set_position (GTK_EDITABLE (entry), -1);
                         gtk_editable_select_region (GTK_EDITABLE (entry), -1, -1);
                 }
-                return TRUE;
+
+                return GDK_EVENT_STOP;
         }
 
         if (event->keyval == GDK_KEY_Return ||
@@ -268,11 +264,11 @@ sidebar_entry_key_press_event_cb (GtkEntry    *entry,
 
                         g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
 
-                        return TRUE;
+                        return GDK_EVENT_STOP;
                 }
         }
 
-        return FALSE;
+        return GDK_EVENT_PROPAGATE;
 }
 
 static void
@@ -294,7 +290,7 @@ sidebar_entry_changed_cb (GtkEntry  *entry,
 }
 
 static gboolean
-sidebar_complete_idle (DhSidebar *sidebar)
+sidebar_complete_idle_cb (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
         const gchar  *str;
@@ -314,9 +310,9 @@ sidebar_complete_idle (DhSidebar *sidebar)
                 g_free (completed);
         }
 
-        priv->idle_complete = 0;
+        priv->idle_complete_id = 0;
 
-        return FALSE;
+        return G_SOURCE_REMOVE;
 }
 
 static void
@@ -328,9 +324,9 @@ sidebar_entry_text_inserted_cb (GtkEntry    *entry,
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
-        if (!priv->idle_complete)
-                priv->idle_complete =
-                        g_idle_add ((GSourceFunc) sidebar_complete_idle, sidebar);
+        if (priv->idle_complete_id == 0)
+                priv->idle_complete_id =
+                        g_idle_add ((GSourceFunc) sidebar_complete_idle_cb, sidebar);
 }
 
 /******************************************************************************/
@@ -461,6 +457,26 @@ dh_sidebar_new (DhBookManager *book_manager)
                                          NULL));
 }
 
+static void
+dh_sidebar_dispose (GObject *object)
+{
+        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (DH_SIDEBAR (object));
+
+        g_clear_object (&priv->book_manager);
+
+        if (priv->idle_complete_id != 0) {
+                g_source_remove (priv->idle_complete_id);
+                priv->idle_complete_id = 0;
+        }
+
+        if (priv->idle_filter_id != 0) {
+                g_source_remove (priv->idle_filter_id);
+                priv->idle_filter_id = 0;
+        }
+
+        G_OBJECT_CLASS (dh_sidebar_parent_class)->dispose (object);
+}
+
 static void
 dh_sidebar_finalize (GObject *object)
 {
@@ -471,16 +487,6 @@ dh_sidebar_finalize (GObject *object)
         G_OBJECT_CLASS (dh_sidebar_parent_class)->finalize (object);
 }
 
-static void
-dh_sidebar_dispose (GObject *object)
-{
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (DH_SIDEBAR (object));
-
-        g_clear_object (&priv->book_manager);
-
-        G_OBJECT_CLASS (dh_sidebar_parent_class)->dispose (object);
-}
-
 static void
 dh_sidebar_init (DhSidebar *sidebar)
 {
@@ -602,22 +608,29 @@ dh_sidebar_constructed (GObject *object)
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
         /* Setup book manager */
-        g_signal_connect (priv->book_manager,
-                          "book-created",
-                          G_CALLBACK (sidebar_book_created_or_enabled_cb),
-                          sidebar);
-        g_signal_connect (priv->book_manager,
-                          "book-deleted",
-                          G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
-                          sidebar);
-        g_signal_connect (priv->book_manager,
-                          "book-enabled",
-                          G_CALLBACK (sidebar_book_created_or_enabled_cb),
-                          sidebar);
-        g_signal_connect (priv->book_manager,
-                          "book-disabled",
-                          G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
-                          sidebar);
+        g_signal_connect_object (priv->book_manager,
+                                 "book-created",
+                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
+                                 sidebar,
+                                 0);
+
+        g_signal_connect_object (priv->book_manager,
+                                 "book-deleted",
+                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
+                                 sidebar,
+                                 0);
+
+        g_signal_connect_object (priv->book_manager,
+                                 "book-enabled",
+                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
+                                 sidebar,
+                                 0);
+
+        g_signal_connect_object (priv->book_manager,
+                                 "book-disabled",
+                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
+                                 sidebar,
+                                 0);
 
         priv->book_tree = dh_book_tree_new (priv->book_manager);
         gtk_widget_show (priv->book_tree);
@@ -640,8 +653,8 @@ dh_sidebar_class_init (DhSidebarClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->finalize = dh_sidebar_finalize;
         object_class->dispose = dh_sidebar_dispose;
+        object_class->finalize = dh_sidebar_finalize;
         object_class->get_property = dh_sidebar_get_property;
         object_class->set_property = dh_sidebar_set_property;
         object_class->constructed = dh_sidebar_constructed;
